@@ -19,7 +19,9 @@
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 #include <thunar-vfs/thunar-vfs.h>
+#include <libxfcegui4/libxfcegui4.h>
 #include <libexif/exif-data.h>
+#include <dbus/dbus-glib.h>
 
 #include "navigator.h"
 #include "thumbnail_viewer.h"
@@ -39,6 +41,9 @@ struct _RsttoMainWindowPriv
     GList *menu_apps_list;
     gdouble zoom_factor;
 
+    DBusGConnection *connection;
+    DBusGProxy *filemanager_proxy;
+
     struct {
         GtkWidget *main_vbox;
         GtkWidget *s_window;
@@ -49,6 +54,8 @@ struct _RsttoMainWindowPriv
         GtkOrientation thumbnail_viewer_orientation;
         gboolean       thumbnail_viewer_visibility;
         gboolean       toolbar_visibility;
+        gint           max_cache_size;
+        gdouble        slideshow_timeout;
     } settings;
 
     struct {
@@ -65,6 +72,8 @@ struct _RsttoMainWindowPriv
                 GtkWidget *menu_item_clear;
             } recently;
             GtkWidget *menu_item_separator_1;
+            GtkWidget *menu_item_file_properties;
+            GtkWidget *menu_item_separator_2;
             GtkWidget *menu_item_close;
             GtkWidget *menu_item_quit;
         } file;
@@ -77,6 +86,7 @@ struct _RsttoMainWindowPriv
                 GtkWidget *menu;
                 GtkWidget *menu_item_empty;
             } open_with;
+            GtkWidget *menu_item_preferences;
         } edit;
 
         GtkWidget *menu_item_view;
@@ -196,9 +206,13 @@ cb_rstto_main_window_clear_recent(GtkWidget *widget, RsttoMainWindow *window);
 static void
 cb_rstto_main_window_close(GtkWidget *widget, RsttoMainWindow *window);
 static void
+cb_rstto_main_window_file_properties(GtkWidget *widget, RsttoMainWindow *window);
+static void
 cb_rstto_main_window_quit(GtkWidget *widget, RsttoMainWindow *window);
 static void
 cb_rstto_main_window_about(GtkWidget *widget, RsttoMainWindow *window);
+static void
+cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window);
 
 static void
 cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoNavigatorEntry *entry, RsttoMainWindow *window);
@@ -277,6 +291,8 @@ rstto_main_window_init(RsttoMainWindow *window)
     window->priv->menus.file.menu_item_open_folder = gtk_menu_item_new_with_mnemonic(_("O_pen Folder"));
     window->priv->menus.file.menu_item_open_recently = gtk_menu_item_new_with_mnemonic(_("_Recently used"));
     window->priv->menus.file.menu_item_separator_1 = gtk_separator_menu_item_new();
+    window->priv->menus.file.menu_item_file_properties = gtk_image_menu_item_new_from_stock(GTK_STOCK_PROPERTIES, accel_group);
+    window->priv->menus.file.menu_item_separator_2 = gtk_separator_menu_item_new();
     window->priv->menus.file.menu_item_close = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, accel_group);
     window->priv->menus.file.menu_item_quit = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, accel_group);
 
@@ -285,10 +301,13 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_open_folder);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_open_recently);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_separator_1);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_file_properties);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_separator_2);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_close);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_quit);
 
     gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, FALSE);
+    gtk_widget_set_sensitive(window->priv->menus.file.menu_item_file_properties, FALSE);
 
     window->priv->menus.file.recently.menu = gtk_recent_chooser_menu_new_for_manager(GTK_RECENT_MANAGER(window->priv->manager));
     window->priv->menus.file.recently.menu_item_clear = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR, accel_group);
@@ -308,9 +327,11 @@ rstto_main_window_init(RsttoMainWindow *window)
     window->priv->menus.edit.menu = gtk_menu_new();
     gtk_menu_set_accel_group(GTK_MENU(window->priv->menus.edit.menu), accel_group);
     window->priv->menus.edit.menu_item_open_with = gtk_menu_item_new_with_mnemonic(_("Open with..."));
+    window->priv->menus.edit.menu_item_preferences = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
 
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(window->priv->menus.menu_item_edit), window->priv->menus.edit.menu);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.edit.menu), window->priv->menus.edit.menu_item_open_with);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.edit.menu), window->priv->menus.edit.menu_item_preferences);
     
     window->priv->menus.edit.open_with.menu = gtk_menu_new();
     window->priv->menus.edit.open_with.menu_item_empty = gtk_menu_item_new_with_label(_("No applications available"));
@@ -514,6 +535,13 @@ rstto_main_window_init(RsttoMainWindow *window)
 
     rstto_picture_viewer_set_menu(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer),
                                   GTK_MENU(window->priv->menus._picture_viewer.menu));
+/* D-Bus stuff */
+
+    window->priv->connection = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
+    window->priv->filemanager_proxy = dbus_g_proxy_new_for_name(window->priv->connection,
+                                                                "org.xfce.FileManager",
+                                                                "/org/xfce/FileManager",
+                                                                "org.xfce.FileManager");
 
 /* Connect signals */
     
@@ -567,6 +595,9 @@ rstto_main_window_init(RsttoMainWindow *window)
     g_signal_connect(G_OBJECT(window->priv->menus.file.recently.menu),
             "item-activated",
             G_CALLBACK(cb_rstto_main_window_open_recent), window);
+    g_signal_connect(window->priv->menus.file.menu_item_file_properties, 
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_file_properties), window);
     g_signal_connect(window->priv->menus.file.menu_item_quit, 
             "activate",
             G_CALLBACK(cb_rstto_main_window_quit), window);
@@ -577,6 +608,10 @@ rstto_main_window_init(RsttoMainWindow *window)
             "activate",
             G_CALLBACK(cb_rstto_main_window_about), window);
 
+    g_signal_connect(window->priv->menus.edit.menu_item_preferences, 
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_preferences), window);
+
     g_signal_connect(window->priv->menus.go.menu_item_next,
             "activate",
             G_CALLBACK(cb_rstto_main_window_next), window);
@@ -788,6 +823,31 @@ rstto_main_window_get_recent_manager (RsttoMainWindow *window)
     return window->priv->manager;
 }
 
+gdouble
+rstto_main_window_get_slideshow_timeout (RsttoMainWindow *window)
+{
+    return window->priv->settings.slideshow_timeout;
+}
+
+gint
+rstto_main_window_get_max_cache_size (RsttoMainWindow *window)
+{
+    return window->priv->settings.max_cache_size;
+}
+
+void
+rstto_main_window_set_slideshow_timeout (RsttoMainWindow *window, gdouble timeout)
+{
+    window->priv->settings.slideshow_timeout = timeout;
+    rstto_navigator_set_timeout(window->priv->navigator, timeout);
+}
+
+void
+rstto_main_window_set_max_cache_size (RsttoMainWindow *window, gint max_cache_size)
+{
+    window->priv->settings.max_cache_size = max_cache_size;
+}
+
 /* CALLBACK FUNCTIONS */
 
 static void
@@ -924,7 +984,6 @@ cb_rstto_main_window_play(GtkWidget *widget, RsttoMainWindow *window)
     gtk_menu_shell_insert(GTK_MENU_SHELL(window->priv->menus.go.menu), window->priv->menus.go.menu_item_pause, 5);
     gtk_widget_show_all(window->priv->menus.go.menu_item_pause);
     rstto_navigator_set_running(RSTTO_NAVIGATOR(window->priv->navigator), TRUE);
-    rstto_picture_viewer_set_timeout(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), FALSE);
 }
 
 static void
@@ -935,7 +994,77 @@ cb_rstto_main_window_pause(GtkWidget *widget, RsttoMainWindow *window)
     gtk_menu_shell_insert(GTK_MENU_SHELL(window->priv->menus.go.menu), window->priv->menus.go.menu_item_play, 5);
     gtk_widget_show_all(window->priv->menus.go.menu_item_play);
     rstto_navigator_set_running(RSTTO_NAVIGATOR(window->priv->navigator), FALSE);
-    rstto_picture_viewer_set_timeout(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), TRUE);
+}
+
+static void
+cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
+{
+    GtkWidget *dialog = xfce_titled_dialog_new_with_buttons(_("Image viewer Preferences"),
+                                                    GTK_WINDOW(window),
+                                                    0,
+                                                    GTK_STOCK_CANCEL,
+                                                    GTK_RESPONSE_CANCEL,
+                                                    GTK_STOCK_OK,
+                                                    GTK_RESPONSE_OK,
+                                                    NULL);
+    gtk_window_set_icon_name(GTK_WINDOW(dialog), "ristretto");
+
+    GtkWidget *notebook = gtk_notebook_new();
+
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook,  TRUE, TRUE, 0);
+
+    GtkWidget *main_vbox = gtk_vbox_new(FALSE, 0);
+    GtkWidget *main_lbl = gtk_label_new(_("Behaviour"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), main_vbox, main_lbl);
+
+    GtkWidget *slideshow_frame = gtk_frame_new(_("Slideshow:"));
+    GtkWidget *preload_frame = gtk_frame_new(_("Preload:"));
+
+    GtkWidget *slideshow_vbox = gtk_vbox_new(FALSE, 0);
+    GtkWidget *slideshow_lbl = gtk_label_new(_("The time individual images are displayed during a slideshow\n(in seconds)"));
+    GtkWidget *slideshow_hscale = gtk_hscale_new_with_range(1, 60, 1);
+
+
+    GtkWidget *preload_vbox = gtk_vbox_new(FALSE, 0);
+    GtkWidget *preload_lbl = gtk_label_new(_("Preload images during slideshow\n(uses more memory)"));
+    GtkWidget *preload_check = gtk_check_button_new_with_mnemonic(_("_Preload images"));
+
+    gtk_misc_set_alignment(GTK_MISC(slideshow_lbl), 0, 0.5);
+    gtk_misc_set_alignment(GTK_MISC(preload_lbl), 0, 0.5);
+
+    gtk_misc_set_padding(GTK_MISC(slideshow_lbl), 2, 2);
+    gtk_misc_set_padding(GTK_MISC(preload_lbl), 2, 2);
+    
+    gtk_range_set_value(GTK_RANGE(slideshow_hscale), window->priv->settings.slideshow_timeout / 1000);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preload_check), window->priv->navigator->preload);
+
+    gtk_box_pack_start(GTK_BOX(slideshow_vbox), slideshow_lbl, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(slideshow_vbox), slideshow_hscale, FALSE, TRUE, 0);
+
+    gtk_box_pack_start(GTK_BOX(preload_vbox), preload_lbl, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(preload_vbox), preload_check, FALSE, TRUE, 0);
+
+    gtk_container_add(GTK_CONTAINER(slideshow_frame), slideshow_vbox);
+    gtk_container_add(GTK_CONTAINER(preload_frame), preload_vbox);
+
+    gtk_box_pack_start(GTK_BOX(main_vbox), slideshow_frame, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(main_vbox), preload_frame, FALSE, TRUE, 0);
+
+    gtk_widget_show_all(notebook);
+
+    gint result = gtk_dialog_run(GTK_DIALOG(dialog));
+
+    switch (result)
+    {
+        case GTK_RESPONSE_OK:
+            rstto_main_window_set_slideshow_timeout(window, gtk_range_get_value(GTK_RANGE(slideshow_hscale)) * 1000);
+            window->priv->navigator->preload = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(preload_check));
+        default:
+            break;
+    }
+
+    gtk_widget_destroy(dialog);
 }
 
 static void
@@ -999,12 +1128,7 @@ cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
             gchar *file_media = thunar_vfs_mime_info_get_media(info->mime_info);
             if(!strcmp(file_media, "image"))
             {
-                if(rstto_navigator_get_is_album(window->priv->navigator) == TRUE)
-                {
-                    rstto_navigator_clear(window->priv->navigator);
-                }
-                rstto_navigator_set_is_album(window->priv->navigator, FALSE);
-                RsttoNavigatorEntry *entry = rstto_navigator_entry_new(info);
+                RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
                 rstto_navigator_add (window->priv->navigator, entry);
                 gchar *uri = thunar_vfs_path_dup_uri(info->path);
                 gtk_recent_manager_add_item(window->priv->manager, uri);
@@ -1041,7 +1165,6 @@ cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
     if(response == GTK_RESPONSE_OK)
     {
         rstto_navigator_clear(window->priv->navigator);
-        rstto_navigator_set_is_album(window->priv->navigator, TRUE);
         const gchar *dir_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
         GDir *dir = g_dir_open(dir_name, 0, NULL);
         if (dir)
@@ -1057,7 +1180,7 @@ cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
                     gchar *file_media = thunar_vfs_mime_info_get_media(info->mime_info);
                     if(!strcmp(file_media, "image"))
                     {
-                        RsttoNavigatorEntry *entry = rstto_navigator_entry_new(info);
+                        RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
                         rstto_navigator_add (window->priv->navigator, entry);
                     }
                     g_free(file_media);
@@ -1088,18 +1211,12 @@ cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *win
         {
             if(strcmp(thunar_vfs_mime_info_get_name(info->mime_info), "inode/directory"))
             {
-                if(rstto_navigator_get_is_album(window->priv->navigator) == TRUE)
-                {
-                    rstto_navigator_clear(window->priv->navigator);
-                }
-                rstto_navigator_set_is_album(window->priv->navigator, FALSE);
-                RsttoNavigatorEntry *entry = rstto_navigator_entry_new(info);
+                RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
                 rstto_navigator_add (window->priv->navigator, entry);
             }
             else
             {
                 rstto_navigator_clear(window->priv->navigator);
-                rstto_navigator_set_is_album(window->priv->navigator, TRUE);
                 gchar *dir_path = thunar_vfs_path_dup_string(info->path);
                 GDir *dir = g_dir_open(dir_path, 0, NULL);
                 const gchar *filename = g_dir_read_name(dir);
@@ -1113,7 +1230,7 @@ cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *win
                         gchar *file_media = thunar_vfs_mime_info_get_media(file_info->mime_info);
                         if(!strcmp(file_media, "image"))
                         {
-                            RsttoNavigatorEntry *entry = rstto_navigator_entry_new(file_info);
+                            RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, file_info);
                             rstto_navigator_add (window->priv->navigator, entry);
                         }
                         g_free(file_media);
@@ -1170,6 +1287,32 @@ cb_rstto_main_window_close(GtkWidget *widget, RsttoMainWindow *window)
     }
 }
 
+static void
+cb_rstto_main_window_file_properties(GtkWidget *widget, RsttoMainWindow *window)
+{
+    GError *error = NULL;
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(window->priv->navigator);
+    if (entry)
+    {
+        ThunarVfsInfo *info = rstto_navigator_entry_get_info(entry);
+        if(info)
+        {
+            gchar *uri = thunar_vfs_path_dup_uri(info->path);
+            if(dbus_g_proxy_call(window->priv->filemanager_proxy,
+                                 "DisplayFileProperties",
+                                 &error,
+                                 G_TYPE_STRING, uri,
+                                 G_TYPE_STRING, "",
+                                 G_TYPE_INVALID,
+                                 G_TYPE_INVALID) == FALSE)
+            {
+                g_warning("%s", error->message);
+            }
+            g_free(uri);
+        }
+    }
+}
+
 static void
 cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoNavigatorEntry *entry, RsttoMainWindow *window)
 {
@@ -1182,6 +1325,7 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
         info = rstto_navigator_entry_get_info(entry);
         filename = info->display_name;
         gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, TRUE);
+        gtk_widget_set_sensitive(window->priv->menus.file.menu_item_file_properties, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_first, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_last, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_previous, TRUE);
@@ -1286,6 +1430,8 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
             gtk_widget_set_sensitive(window->priv->menus.go.menu_item_next, FALSE);
             gtk_widget_set_sensitive(window->priv->menus.go.menu_item_play, FALSE);
             gtk_widget_set_sensitive(window->priv->menus.go.menu_item_pause, FALSE);
+            gtk_widget_set_sensitive(window->priv->menus.file.menu_item_file_properties, FALSE);
+
             gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_next), FALSE);
             gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_previous), FALSE);
             gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_zoom_in), FALSE);
