@@ -24,25 +24,34 @@
 #include "navigator.h"
 #include "picture_viewer.h"
 
+#ifndef RSTTO_PICTURE_VIEWER_DRAG_MOTION_TIMEOUT
+#define RSTTO_PICTURE_VIEWER_DRAG_MOTION_TIMEOUT 160
+#endif
+
 struct _RsttoPictureViewerPriv
 {
-    GdkPixbufLoader  *loader;
-    GdkPixbufAnimation *animation;
-    GdkPixbufAnimationIter *iter;
-    GIOChannel       *io_channel;
+    RsttoNavigator   *navigator;
+    RsttoNavigatorEntry *entry;
+
     GdkPixbuf        *src_pixbuf;
     GdkPixbuf        *dst_pixbuf; /* The pixbuf which ends up on screen */
-    RsttoNavigator   *navigator;
     void             (*cb_value_changed)(GtkAdjustment *, RsttoPictureViewer *);
     gboolean          show_border;
-    GTimer           *timer;
-    gint              timeout_id;
-    gboolean          timeout;
     struct
     {
         gdouble x;
         gdouble y;
+        gdouble current_x;
+        gdouble current_y;
+        gint h_val;
+        gint v_val;
+        gint32 time;
+        gint idle_id;
     } motion;
+    struct
+    {
+        gint idle_id;
+    } zoom;
     GtkMenu          *menu;
 };
 
@@ -53,6 +62,9 @@ rstto_picture_viewer_class_init(RsttoPictureViewerClass *);
 static void
 rstto_picture_viewer_destroy(GtkObject *object);
 
+static gboolean 
+cb_rstto_picture_viewer_queued_redraw (RsttoPictureViewer *viewer);
+
 static void
 rstto_picture_viewer_size_request(GtkWidget *, GtkRequisition *);
 static void
@@ -72,6 +84,9 @@ rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *, GtkAdjustment
 
 static void
 cb_rstto_picture_viewer_nav_iter_changed(RsttoNavigator *, gint , RsttoNavigatorEntry *, RsttoPictureViewer *);
+static void
+cb_rstto_picture_viewer_nav_entry_modified(RsttoNavigator *, RsttoNavigatorEntry *, RsttoPictureViewer *);
+
 static void
 cb_rstto_picture_viewer_value_changed(GtkAdjustment *, RsttoPictureViewer *);
 static void
@@ -87,20 +102,6 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
 static void
 cb_rstto_picture_viewer_popup_menu (RsttoPictureViewer *viewer, gboolean user_data);
 
-static gboolean
-cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer);
-
-static gboolean
-cb_rstto_picture_viewer_read_file(GIOChannel *io_channel, GIOCondition cond, RsttoPictureViewer *viewer);
-
-static void
-cb_rstto_picture_viewer_area_prepared(GdkPixbufLoader *loader, RsttoPictureViewer *viewer);
-static void
-cb_rstto_picture_viewer_area_updated(GdkPixbufLoader *loader, gint x, gint y, gint width, gint height, RsttoPictureViewer *viewer);
-static void
-cb_rstto_picture_viewer_closed(GdkPixbufLoader *loader, RsttoPictureViewer *viewer);
-
-
 static GtkWidgetClass *parent_class = NULL;
 
 GType
@@ -144,8 +145,6 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
                            GDK_BUTTON1_MOTION_MASK);
 
     viewer->priv->show_border = TRUE;
-    viewer->priv->timer = g_timer_new();
-    viewer->priv->timeout = TRUE;
 
     g_signal_connect(G_OBJECT(viewer), "scroll_event", G_CALLBACK(cb_rstto_picture_viewer_scroll_event), NULL);
     g_signal_connect(G_OBJECT(viewer), "button_press_event", G_CALLBACK(cb_rstto_picture_viewer_button_press_event), NULL);
@@ -212,6 +211,7 @@ rstto_picture_viewer_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
     }
 
     rstto_picture_viewer_refresh(viewer);
+    rstto_picture_viewer_paint(GTK_WIDGET(viewer));
 }
 
 static void
@@ -248,10 +248,8 @@ rstto_picture_viewer_realize(GtkWidget *widget)
 static gboolean
 rstto_picture_viewer_expose(GtkWidget *widget, GdkEventExpose *event)
 {
-    if(rstto_picture_viewer_refresh(RSTTO_PICTURE_VIEWER(widget)))
-    {
-        rstto_picture_viewer_paint(widget);
-    }
+    rstto_picture_viewer_refresh(RSTTO_PICTURE_VIEWER(widget));
+    rstto_picture_viewer_paint(widget);
     return FALSE;
 }
 
@@ -406,6 +404,10 @@ cb_rstto_picture_viewer_value_changed(GtkAdjustment *adjustment, RsttoPictureVie
         width = (gdouble)gdk_pixbuf_get_width(viewer->priv->src_pixbuf);
         height = (gdouble)gdk_pixbuf_get_height(viewer->priv->src_pixbuf);
     }
+    if (viewer->priv->src_pixbuf == NULL)
+    {
+        return;
+    }
     GdkPixbuf *tmp_pixbuf = NULL;
     if (viewer->vadjustment && viewer->hadjustment)
     {
@@ -471,6 +473,7 @@ rstto_picture_viewer_new(RsttoNavigator *navigator)
     widget = g_object_new(RSTTO_TYPE_PICTURE_VIEWER, NULL);
     RSTTO_PICTURE_VIEWER(widget)->priv->navigator = navigator;
     g_signal_connect(G_OBJECT(navigator), "iter-changed", G_CALLBACK(cb_rstto_picture_viewer_nav_iter_changed), widget);
+    g_signal_connect(G_OBJECT(navigator), "entry-modified", G_CALLBACK(cb_rstto_picture_viewer_nav_entry_modified), widget);
 
     return widget;
 }
@@ -481,13 +484,47 @@ rstto_picture_viewer_set_scale(RsttoPictureViewer *viewer, gdouble scale)
     RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
     if (entry)
     {
+        gdouble old_scale = rstto_navigator_entry_get_scale(entry);
+
+        gdouble width = (gdouble)gdk_pixbuf_get_width(viewer->priv->src_pixbuf);
+        gdouble height = (gdouble)gdk_pixbuf_get_height(viewer->priv->src_pixbuf);
+
         rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
         rstto_navigator_entry_set_scale(entry, scale);
 
-        if(rstto_picture_viewer_refresh(viewer))
+        viewer->hadjustment->upper = width * scale;
+        gtk_adjustment_changed(viewer->hadjustment);
+
+        viewer->vadjustment->upper = height * scale;
+        gtk_adjustment_changed(viewer->vadjustment);
+
+        viewer->hadjustment->value = (((viewer->hadjustment->value +
+                                      (viewer->hadjustment->page_size / 2)) *
+                                       scale) / old_scale) - (viewer->hadjustment->page_size / 2);
+        viewer->vadjustment->value = (((viewer->vadjustment->value +
+                                      (viewer->vadjustment->page_size / 2)) *
+                                       scale) / old_scale) - (viewer->vadjustment->page_size / 2);
+
+        if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
         {
-            rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+            viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
         }
+        if(viewer->hadjustment->value < viewer->hadjustment->lower)
+        {
+            viewer->hadjustment->value = viewer->hadjustment->lower;
+        }
+        if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
+        {
+            viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
+        }
+        if(viewer->vadjustment->value < viewer->vadjustment->lower)
+        {
+            viewer->vadjustment->value = viewer->vadjustment->lower;
+        }
+
+        gtk_adjustment_value_changed(viewer->hadjustment);
+        gtk_adjustment_value_changed(viewer->vadjustment);
+
     }
 }
 
@@ -602,6 +639,11 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
                     viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
                     hadjustment_changed = TRUE;
                 }
+                if(viewer->hadjustment->value < viewer->hadjustment->lower)
+                {
+                    viewer->hadjustment->value = viewer->hadjustment->lower;
+                    hadjustment_changed = TRUE;
+                }
             }
             if(viewer->vadjustment)
             {
@@ -615,6 +657,11 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
                     viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
                     vadjustment_changed = TRUE;
                 }
+                if(viewer->vadjustment->value < viewer->vadjustment->lower)
+                {
+                    viewer->vadjustment->value = viewer->vadjustment->lower;
+                    vadjustment_changed = TRUE;
+                }
             }
 
 
@@ -679,119 +726,37 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
     return changed;
 }
 
-static gboolean
-cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer)
-{
-    if (viewer->priv->iter)
-    {
-        if(gdk_pixbuf_animation_iter_advance(viewer->priv->iter, NULL))
-        {
-            /* Cleanup old image */
-            if (viewer->priv->src_pixbuf)
-            {
-                gdk_pixbuf_unref(viewer->priv->src_pixbuf);
-                viewer->priv->src_pixbuf = NULL;
-            }
-            RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
-
-            if (entry)
-            {
-                GdkPixbuf *src_pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(viewer->priv->iter);
-                if (src_pixbuf)
-                {
-                    viewer->priv->src_pixbuf = gdk_pixbuf_rotate_simple(src_pixbuf, rstto_navigator_entry_get_rotation(entry));
-                    if (rstto_navigator_entry_get_flip(entry, FALSE))
-                    {
-                        src_pixbuf = viewer->priv->src_pixbuf;
-                        viewer->priv->src_pixbuf = gdk_pixbuf_flip(src_pixbuf, FALSE);
-                        gdk_pixbuf_unref(src_pixbuf);
-                    }
-
-                    if (rstto_navigator_entry_get_flip(entry, TRUE))
-                    {
-                        src_pixbuf = viewer->priv->src_pixbuf;
-                        viewer->priv->src_pixbuf = gdk_pixbuf_flip(src_pixbuf, TRUE);
-                        gdk_pixbuf_unref(src_pixbuf);
-                    }
-                }
-            }
-
-            rstto_picture_viewer_refresh(viewer);
-            rstto_picture_viewer_paint(GTK_WIDGET(viewer));
-
-            gint time = gdk_pixbuf_animation_iter_get_delay_time(viewer->priv->iter);
-            if (time != -1)
-            {
-                viewer->priv->timeout_id = g_timeout_add(time, (GSourceFunc)cb_rstto_picture_viewer_update_image, viewer);
-            }
-
-            return FALSE;
-        }
-    }
-    return TRUE;
-}
-
 static void
 cb_rstto_picture_viewer_nav_iter_changed(RsttoNavigator *nav, gint nr, RsttoNavigatorEntry *entry, RsttoPictureViewer *viewer)
 {
-    GtkWidget *widget = GTK_WIDGET(viewer);
+    viewer->priv->entry = entry;
     if(entry)
-    { 
-        if (GTK_WIDGET_REALIZED(widget))
-        {
-            GdkCursor *cursor = gdk_cursor_new(GDK_WATCH);
-            gdk_window_set_cursor(widget->window, cursor);
-            gdk_cursor_unref(cursor);
-        }
-        if (viewer->priv->loader)
-        {
-            g_signal_handlers_disconnect_by_func(viewer->priv->loader , cb_rstto_picture_viewer_area_prepared, viewer);
-            g_signal_handlers_disconnect_by_func(viewer->priv->loader , cb_rstto_picture_viewer_area_updated, viewer);
-            gdk_pixbuf_loader_close(viewer->priv->loader, NULL);
-            g_object_unref(viewer->priv->loader);
-            if (viewer->priv->iter)
-            {
-                viewer->priv->iter = NULL;
-            }
-            if(viewer->priv->animation)
-            {
-                g_object_unref(viewer->priv->animation);
-                viewer->priv->animation = NULL;
-            }
-            if(viewer->priv->src_pixbuf)
-            {
-                gdk_pixbuf_unref(viewer->priv->src_pixbuf);
-                viewer->priv->src_pixbuf = NULL;
-            }
-        }
-        viewer->priv->loader = gdk_pixbuf_loader_new();
-
-        g_signal_connect(viewer->priv->loader, "area-prepared", G_CALLBACK(cb_rstto_picture_viewer_area_prepared), viewer);
-        g_signal_connect(viewer->priv->loader, "area-updated", G_CALLBACK(cb_rstto_picture_viewer_area_updated), viewer);
-        g_signal_connect(viewer->priv->loader, "closed", G_CALLBACK(cb_rstto_picture_viewer_closed), viewer);
-        ThunarVfsInfo *info = rstto_navigator_entry_get_info(entry);
-        gchar *path = thunar_vfs_path_dup_string(info->path);
-
-        viewer->priv->io_channel = g_io_channel_new_file(path, "r", NULL);
-        g_io_channel_set_encoding(viewer->priv->io_channel, NULL, NULL);
-        g_io_add_watch(viewer->priv->io_channel, G_IO_IN | G_IO_PRI, (GIOFunc)cb_rstto_picture_viewer_read_file, viewer);
+    {
+        rstto_navigator_entry_load_image(entry, FALSE);
     }
     else
     {
+        rstto_picture_viewer_refresh(viewer);
+        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+    }
+}
+
+static void
+cb_rstto_picture_viewer_nav_entry_modified(RsttoNavigator *nav, RsttoNavigatorEntry *entry, RsttoPictureViewer *viewer)
+{
+    if (entry == viewer->priv->entry)
+    {
+        if(viewer->priv->src_pixbuf)
+        {
+            gdk_pixbuf_unref(viewer->priv->src_pixbuf);
+        }
+        viewer->priv->src_pixbuf = rstto_navigator_entry_get_pixbuf(entry);
         if (viewer->priv->src_pixbuf)
         {
-            g_object_unref(viewer->priv->src_pixbuf);
-            viewer->priv->src_pixbuf = NULL;
-        }
-        if (viewer->priv->dst_pixbuf)
-        {
-            g_object_unref(viewer->priv->dst_pixbuf);
-            viewer->priv->dst_pixbuf = NULL;
-        }
-        if (GTK_WIDGET_REALIZED(widget))
-        {
-            rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+            gdk_pixbuf_ref(viewer->priv->src_pixbuf);
         }
+        rstto_picture_viewer_refresh(viewer);
+        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
     }
 }
 
@@ -806,188 +771,21 @@ cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *viewer, GdkEventScroll
         case GDK_SCROLL_LEFT:
             if (scale <= 0.05)
                 return;
-            rstto_navigator_entry_set_scale(entry, scale / 1.2);
-            rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
+            if (viewer->priv->motion.idle_id > 0)
+            {
+                g_source_remove(viewer->priv->motion.idle_id);
+            }
             break;
         case GDK_SCROLL_DOWN:
         case GDK_SCROLL_RIGHT:
             if (scale >= 16)
                 return;
-            viewer->vadjustment->value = ((viewer->vadjustment->value + event->y) - viewer->vadjustment->page_size/2) * 1.2;
-            viewer->hadjustment->value = ((viewer->hadjustment->value + event->x) - viewer->hadjustment->page_size/2) * 1.2;
-
-            rstto_navigator_entry_set_scale(entry, scale * 1.2);
-            rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
+            if (viewer->priv->motion.idle_id > 0)
+            {
+                g_source_remove(viewer->priv->motion.idle_id);
+            }
             break;
     }
-
-    if(rstto_picture_viewer_refresh(viewer))
-    {
-        rstto_picture_viewer_paint(GTK_WIDGET(viewer));
-    }
-}
-
-void
-rstto_picture_viewer_set_timeout(RsttoPictureViewer *viewer, gboolean timeout)
-{
-    viewer->priv->timeout = timeout;
-}
-
-static gboolean
-cb_rstto_picture_viewer_read_file(GIOChannel *io_channel, GIOCondition cond, RsttoPictureViewer *viewer)
-{
-    gchar buffer[1024];
-    gsize bytes_read = 0;
-    GError *error = NULL;
-
-
-    GIOStatus status;
-    if (viewer->priv->io_channel == io_channel)
-    {
-        status = g_io_channel_read_chars(io_channel, buffer, 1024, &bytes_read,  &error);
-
-        switch (status)
-        {
-            case G_IO_STATUS_NORMAL:
-                if(gdk_pixbuf_loader_write(viewer->priv->loader, (const guchar *)buffer, bytes_read, NULL) == FALSE)
-                {
-                    gdk_pixbuf_loader_close(viewer->priv->loader, NULL);
-                    viewer->priv->io_channel = NULL;
-                    return FALSE;
-                }
-                return TRUE;
-                break;
-            case G_IO_STATUS_EOF:
-                gdk_pixbuf_loader_write(viewer->priv->loader, (const guchar *)buffer, bytes_read, NULL);
-                gdk_pixbuf_loader_close(viewer->priv->loader, NULL);
-                viewer->priv->io_channel = NULL;
-                return FALSE;
-                break;
-            case G_IO_STATUS_ERROR:
-                gdk_pixbuf_loader_close(viewer->priv->loader, NULL);
-                viewer->priv->io_channel = NULL;
-                return FALSE;
-                break;
-            case G_IO_STATUS_AGAIN:
-                return TRUE;
-                break;
-        }
-    }
-    g_io_channel_shutdown(io_channel, FALSE, NULL);
-    return FALSE;
-}
-
-static void
-cb_rstto_picture_viewer_area_prepared(GdkPixbufLoader *loader, RsttoPictureViewer *viewer)
-{
-    viewer->priv->animation = gdk_pixbuf_loader_get_animation(loader);
-    viewer->priv->iter = gdk_pixbuf_animation_get_iter(viewer->priv->animation, NULL);
-    if (viewer->priv->src_pixbuf)
-    {
-        gdk_pixbuf_unref(viewer->priv->src_pixbuf);
-        viewer->priv->src_pixbuf = NULL;
-    }
-
-    gint time = gdk_pixbuf_animation_iter_get_delay_time(viewer->priv->iter);
-
-    rstto_picture_viewer_refresh(viewer);
-    rstto_picture_viewer_paint(GTK_WIDGET(viewer));
-
-    if (time != -1)
-    {
-        /* update frame */
-        viewer->priv->timeout_id = g_timeout_add(time, (GSourceFunc)cb_rstto_picture_viewer_update_image, viewer);
-    }   
-    else
-    {
-        viewer->priv->iter = NULL;
-    }
-}
-
-static void
-cb_rstto_picture_viewer_area_updated(GdkPixbufLoader *loader, gint x, gint y, gint width, gint height, RsttoPictureViewer *viewer)
-{
-    if (viewer->priv->iter)
-    {
-        /* Current Frame being updated?! */
-        if (gdk_pixbuf_animation_iter_on_currently_loading_frame(viewer->priv->iter) == TRUE)
-        {
-            /* Is it inside the viewport? */
-            RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
-            gdouble scale = rstto_navigator_entry_get_scale(entry);
-            if (((viewer->vadjustment->value > (y * scale)) && (viewer->vadjustment->value < ((y+height) * scale))) &&
-                ((viewer->hadjustment->value > (x * scale)) && (viewer->hadjustment->value < ((x+width) * scale))))
-            {
-            
-            /* Update */
-            /*
-            gdk_pixbuf_unref(viewer->priv->src_pixbuf);
-            viewer->priv->src_pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(viewer->priv->iter);
-            gdk_pixbuf_ref(viewer->priv->src_pixbuf);
-            rstto_picture_viewer_refresh(viewer);
-            rstto_picture_viewer_paint(viewer);
-            */
-            }
-        }
-    }
-}
-
-static void
-cb_rstto_picture_viewer_closed(GdkPixbufLoader *loader, RsttoPictureViewer *viewer)
-{
-    GtkWidget *widget = GTK_WIDGET(viewer);
-
-    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
-
-    if (viewer->priv->src_pixbuf)
-    {
-        gdk_pixbuf_unref(viewer->priv->src_pixbuf);
-        viewer->priv->src_pixbuf = NULL;
-    }
-
-    if (entry)
-    {
-        GdkPixbuf *pixbuf = NULL;
-
-        if (viewer->priv->iter)
-        {
-            pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(viewer->priv->iter);
-        }
-        else
-        {
-            if (viewer->priv->loader)
-            {
-                pixbuf = gdk_pixbuf_loader_get_pixbuf(viewer->priv->loader);
-            }
-        }
-
-       
-        if (pixbuf != NULL)
-        {
-            viewer->priv->src_pixbuf = gdk_pixbuf_rotate_simple(pixbuf, rstto_navigator_entry_get_rotation(entry));
-            if (rstto_navigator_entry_get_flip(entry, FALSE))
-            {
-                pixbuf = viewer->priv->src_pixbuf;
-                viewer->priv->src_pixbuf = gdk_pixbuf_flip(pixbuf, FALSE);
-                gdk_pixbuf_unref(pixbuf);
-            }
-
-            if (rstto_navigator_entry_get_flip(entry, TRUE))
-            {
-                pixbuf = viewer->priv->src_pixbuf;
-                viewer->priv->src_pixbuf = gdk_pixbuf_flip(pixbuf, TRUE);
-                gdk_pixbuf_unref(pixbuf);
-            }
-        }
-        
-    }
-
-    rstto_picture_viewer_refresh(viewer);
-    rstto_picture_viewer_paint(GTK_WIDGET(viewer));
-    if (GTK_WIDGET_REALIZED(widget))
-    {
-        gdk_window_set_cursor(widget->window, NULL);
-    }
 }
 
 static gboolean 
@@ -997,8 +795,58 @@ cb_rstto_picture_viewer_motion_notify_event (RsttoPictureViewer *viewer,
 {
     if (event->state & GDK_BUTTON1_MASK)
     {
+        if (viewer->priv->motion.time == 0)
+            viewer->priv->motion.time = gtk_get_current_event_time();
+
+        viewer->priv->motion.current_x = event->x;
+        viewer->priv->motion.current_y = event->y;
+
+        if (viewer->priv->motion.idle_id > 0)
+        {
+            g_source_remove(viewer->priv->motion.idle_id);
+        }
+
+        viewer->priv->motion.idle_id = g_idle_add((GSourceFunc)cb_rstto_picture_viewer_queued_redraw, viewer);
     }
-    return TRUE;
+    return FALSE;
+}
+
+static gboolean 
+cb_rstto_picture_viewer_queued_redraw (RsttoPictureViewer *viewer)
+{
+    if (viewer->priv->motion.x != viewer->priv->motion.current_x)
+    {
+        gint val = viewer->hadjustment->value;
+        viewer->hadjustment->value = viewer->priv->motion.h_val + (viewer->priv->motion.x - viewer->priv->motion.current_x);
+        if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
+        {
+            viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
+        }
+        if((viewer->hadjustment->value) < viewer->hadjustment->lower)
+        {
+            viewer->hadjustment->value = viewer->hadjustment->lower;
+        }
+        if (val != viewer->hadjustment->value)
+            gtk_adjustment_value_changed(viewer->hadjustment);
+    }
+
+    if (viewer->priv->motion.y != viewer->priv->motion.current_y)
+    {
+        gint val = viewer->vadjustment->value;
+        viewer->vadjustment->value = viewer->priv->motion.v_val + (viewer->priv->motion.y - viewer->priv->motion.current_y);
+        if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
+        {
+            viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
+        }
+        if((viewer->vadjustment->value) < viewer->vadjustment->lower)
+        {
+            viewer->vadjustment->value = viewer->vadjustment->lower;
+        }
+        if (val != viewer->vadjustment->value)
+            gtk_adjustment_value_changed(viewer->vadjustment);
+    }
+    viewer->priv->motion.idle_id = -1;
+    return FALSE;
 }
 
 static void
@@ -1013,6 +861,8 @@ cb_rstto_picture_viewer_button_press_event (RsttoPictureViewer *viewer, GdkEvent
         
         viewer->priv->motion.x = event->x;
         viewer->priv->motion.y = event->y;
+        viewer->priv->motion.h_val = viewer->hadjustment->value;
+        viewer->priv->motion.v_val = viewer->vadjustment->value;
     }
     if(event->button == 3)
     {
@@ -1035,31 +885,7 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
 {
     if(event->button == 1)
     {
-        /* Move the image around */
         GtkWidget *widget = GTK_WIDGET(viewer);
-        GdkCursor *cursor = gdk_cursor_new(GDK_WATCH);
-        gdk_window_set_cursor(widget->window, cursor);
-        gdk_cursor_unref(cursor);
-
-        if (viewer->priv->motion.x != event->x)
-        {
-            viewer->hadjustment->value += (viewer->priv->motion.x - event->x);
-            if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
-            {
-                viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
-            }
-            gtk_adjustment_value_changed(viewer->hadjustment);
-        }
-        if (viewer->priv->motion.y != event->y)
-        {
-            viewer->vadjustment->value += (viewer->priv->motion.y - event->y);
-            if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
-            {
-                viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
-            }
-            gtk_adjustment_value_changed(viewer->vadjustment);
-        }
-
         gdk_window_set_cursor(widget->window, NULL);
     }
 
